% Rotate the points (xs, ys) by such an angle ph around the origin that the
% sum of the squares of the new y components is minimal ("find the
% principal direction along which the points are distributed"). dph is an
% estimate of the standard deviation of ph.

% Christian Walther, diploma thesis 2005

function [nxs, nys, ph, dph] = autophase(xs, ys)

cxs = xs(:);
cys = ys(:);
ph = 1/2*atan2(2*cxs'*cys, (cxs - cys)'*(cxs + cys));

nxs = cos(ph)*xs + sin(ph)*ys;
nys = -sin(ph)*xs + cos(ph)*ys;

if (mean(nxs(:)) < 0)
    ph = pi - mod(-ph, 2*pi);
    nxs = -nxs;
    nys = -nys;
end

phs = mod(atan2(-nxs, nys), pi);
dph = sqrt(sum((phs(:) - sum(phs(:).*nxs(:))/sum(nxs(:))).^2.*nxs(:))/sum(nxs(:)));
