% Evaluation of measured reflectivity data.

% Christian Walther, diploma thesis 2005

%---- begin input parameters ----

prefix = '../Data/';

%folder = 'ES134-10';
%samples = {'ses1.txt', 'ses2.txt', 'ses3.txt', 'ses4.txt', 'ses5.txt', 'ses6.txt'};
%samples = {'ses4.txt'};

%folder = 'ES134-11';
%samples = {'ses1.txt', 'ses2.txt'};

%folder = 'ES134-13';
%samples = {'ses.txt'};
%samples = {'ses1.txt', 'ses2.txt', 'ses3.txt', 'ses4.txt', 'ses5.txt', 'ses6.txt', 'ses7.txt', 'ses8.txt', 'ses9.txt'};
%samples = {'ses1.txt', 'ses2.txt', 'ses3.txt', 'ses4.txt', 'ses5.txt', 'ses6.txt', 'ses7.txt', 'hr2.txt'};
%samples = {'ses1.txt', 'ses2.txt'};

%folder = 'ES160-1';
%folder = 'ES160-2';
folder = 'ES160-3';
samples = {'ses.txt', 'ses2.txt', 'hr2.txt'};

motorinfluence = [-5e-9, -2e-9];
d_motorinfluence = 4e-9;

spotradius = 13e-6;
d_spotradius = 0.5e-6;


%---- end of input parameters ----


path = [prefix, '/', folder, '/'];

hr = load('-ascii', [path, 'hr.txt']);
short = load('-ascii', [path, 'short.txt']);

clear seslongresfitr seslongresr d_seslongresr sesdistance

for i = 1:length(samples)
    %This loop (the ability to handle multiple samples) was added late in
    %the development process. Probably many things could be done outside of
    %it.

    ses = load('-ascii', [path, samples{i}]);

    hrpickupx = hr(:, 2);
    hrpickupy = hr(:, 3);
    hrrawx = hr(1:end-1, 4);
    hrrawy = hr(1:end-1, 5);
    hrx = hrrawx - (hrpickupx(1:end-1) + hrpickupx(2:end))/2 + motorinfluence(1);
    hry = hrrawy - (hrpickupy(1:end-1) + hrpickupy(2:end))/2 + motorinfluence(2);

    shortpickupx = short(:, 2);
    shortpickupy = short(:, 3);
    shortrawx = short(1:end-1, 4);
    shortrawy = short(1:end-1, 5);
    shortx = shortrawx - (shortpickupx(1:end-1) + shortpickupx(2:end))/2 + motorinfluence(1);
    shorty = shortrawy - (shortpickupy(1:end-1) + shortpickupy(2:end))/2 + motorinfluence(2);

    sespickupx = ses(:, 2);
    sespickupy = ses(:, 3);
    sesrawx = ses(1:end-1, 4);
    sesrawy = ses(1:end-1, 5);
    sesx = sesrawx - (sespickupx(1:end-1) + sespickupx(2:end))/2 + motorinfluence(1);
    sesy = sesrawy - (sespickupy(1:end-1) + sespickupy(2:end))/2 + motorinfluence(2);


    load('-ascii', [path, 'meas_power.txt']);
    d_meas_power = meas_power(1:end, 2);
    meas_power = meas_power(1:end, 1);
    fluenceperpower = 1/(100e6*pi*spotradius^2);
    fluence = meas_power*fluenceperpower;
    d_fluence = d_meas_power*fluenceperpower;
    d_fluence_global = 3/spotradius*d_spotradius;
    fluence_upper = fluence + d_fluence;
    fluence_lower = max(10^floor(log(min(fluence))/log(10)), fluence - d_fluence);

    shortfitx = (meas_power\shortx)*meas_power;
    shortfity = (meas_power\shorty)*meas_power;
    
    % test of quadratic fit
    %a = [meas_power, meas_power.^2]*([meas_power, meas_power.^2]\[shortx, shorty]); 
    %shortfitx = a(:,1);
    %shortfity = a(:,2);
    
    shortfitm = sqrt(shortfitx.^2 + shortfity.^2);
    [shortm, dummy, shortph, dshortph] = autophase(shortx, shorty);
    shortperpower = meas_power\shortm;
    d_shortm = shortperpower*d_meas_power;

    hrlongresfitx = hrx - shortfitx;
    hrlongresfity = hry - shortfity;
    seslongresfitx = sesx - shortfitx;
    seslongresfity = sesy - shortfity;
    hrlongresx = hrx - shortx;
    hrlongresy = hry - shorty;
    seslongresx = sesx - shortx;
    seslongresy = sesy - shorty;

    hrlongresfitm = autophase(hrlongresfitx, hrlongresfity);
    seslongresfitm = autophase(seslongresfitx, seslongresfity);
    hrlongresm = autophase(hrlongresx, hrlongresy);
    seslongresm = autophase(seslongresx, seslongresy);

    hrlongresfitr = hrlongresfitm./shortfitm;
    seslongresfitr{i} = seslongresfitm./shortfitm;
    hrlongresr = hrlongresm./shortm;
    seslongresr{i} = seslongresm./shortm;

    d_hrlongresfitm = sqrt(d_motorinfluence^2 + d_shortm.^2);

    d_hrlongresr = 1./shortfitm .* sqrt((hrx.^2 + hry.^2)./shortfitm.^2.*d_shortm.^2 + d_motorinfluence^2);
    d_seslongresr{i} = 1./shortfitm .* sqrt((sesx.^2 + sesy.^2)./shortfitm.^2.*d_shortm.^2 + d_motorinfluence^2);

    calib = .9975/(sum(hrlongresfitr./d_hrlongresr)/sum(1./d_hrlongresr));

    hrlongresfitr = calib*hrlongresfitr;
    seslongresfitr{i} = calib*seslongresfitr{i};
    hrlongresr = calib*hrlongresr;
    seslongresr{i} = calib*seslongresr{i};
    d_hrlongresr = calib*d_hrlongresr;
    d_seslongresr{i} = calib*d_seslongresr{i};
    
    
    hrdistance = mod(shortph - atan2(hrlongresfity, hrlongresfitx), 2*pi)/(4*pi)*2.9979e8/100e6;
    sesdistance{i} = mod(shortph - atan2(seslongresfity, seslongresfitx), 2*pi)/(4*pi)*2.9979e8/100e6;
    ddistance = dshortph/(4*pi)*2.9979e8/100e6;
    
end


figure(22);
hold off;
plot(hrx, hry+2e-5, 'bo', hrrawx, hrrawy, 'bv', hrpickupx, hrpickupy, 'bx');
hold on;
%plot((shortm\hrlongresx)*shortm + shortx, (shortm\hrlongresy)*shortm + shorty + 2e-5, 'b^');
%plot((meas_power\hrlongresx)*meas_power + shortfitx, (meas_power\hrlongresy)*meas_power + shortfity + 2e-5, 'bv');
%plot((shortm\hrlongresfitx)*shortm + shortx, (shortm\hrlongresfity)*shortm + shorty + 2e-5, 'mv');
%plot((meas_power\hrlongresfitx)*meas_power + shortfitx, (meas_power\hrlongresfity)*meas_power + shortfity + 2e-5, 'm^');
plot(shortx, shorty+3e-5, 'ro', shortrawx, shortrawy, 'rv', shortpickupx, shortpickupy, 'rx');
plot(sesx, sesy+4e-5, 'go', sesrawx, sesrawy, 'gv', sespickupx, sespickupy, 'gx');

plot(hrlongresx, hrlongresy+1e-5, 'k^');
plot((meas_power\hrlongresx)*meas_power, (meas_power\hrlongresy)*meas_power + 1e-5, 'b^');
plot(seslongresx, seslongresy+5e-5, 'm^');
plot(shortfitx, shortfity+3e-5, 'r+');

plot([hrlongresx + d_hrlongresfitm, hrlongresx - d_hrlongresfitm]', [1;1]*(hrlongresy+1e-5)', 'k-+')

axis equal;
grid on;
hold off;
set(gcf, 'Name', [folder, '/', samples{end}]);


is = [1];
for i = 2:length(fluence)
    if (fluence_lower(i) > fluence_upper(is(end)) && fluence(i) > 2*fluence(is(end)))
        is = [is, i];
    end
end
%fluerrys = 0.900*mod(1:length(is), 2) + 0.902*(1-mod(1:length(is), 2)); %ES134
fluerrys = 0.9925*mod(1:length(is), 2) + 0.9927*(1-mod(1:length(is), 2)); %ES160

figure(23);
clf;
hold on;
plot(fluence*100, hrlongresfitr*100, 'ko', 'MarkerSize', 4);
for i = 1:length(samples)
    plot(fluence*100, seslongresfitr{i}*100, 'o', 'Color', distinctColor(i-1), 'MarkerFaceColor', distinctColor(i-1), 'MarkerSize', 4);
end

plot(fluence*100, hrlongresr*100, 'k^', 'MarkerSize', 4);
for i = 1:length(samples)
    plot(fluence*100, seslongresr{i}*100, '^', 'Color', distinctColor(i-1), 'MarkerFaceColor', distinctColor(i-1), 'MarkerSize', 4);
end
plot(fluence*100*[1,1], [hrlongresfitr + d_hrlongresr, hrlongresfitr - d_hrlongresr]*100, 'k--');
for i = 1:length(samples)
    plot(fluence*100*[1,1], [seslongresfitr{i} + d_seslongresr{i}, seslongresfitr{i} - d_seslongresr{i}]*100, '--', 'Color', distinctColor(i-1));
end
plot(100*fluence(is), fluerrys*100, 'kd', [fluence_lower(is)*100, fluence_upper(is)*100]', [1;1]*fluerrys*100, 'k-+', 'MarkerSize', 4, 'Color', [0, 0, .1])

hold off;
xlabel(sprintf('Fluence [{\\mu}J/cm^2] \\pm %.1f %%', 100*d_fluence_global));
ylabel('Reflectivity [%]');
set(gca, 'XScale', 'Log');
%set(gca, 'YLim', [88, 102]); %ES134
set(gca, 'YLim', [99.2, 100]); %ES160
%set(gca, 'YTick', [80, 85, 90:1:100, 105, 110]);
%set(gca, 'YTick', 80:1:110); %ES134
set(gca, 'YTick', 99:.1:100); %ES160
%set(gca, 'XLim', [1e-2, 3e2]); %ES134
set(gca, 'XLim', [3e-1, 3e2]); %ES160
set(gca, 'Box', 'on');
grid on;
legend([{'HR'}, samples], 'Location', 'SE');
set(gcf, 'Name', folder);


figure(24);
plot(fluence*100, 1000*hrdistance, 'ko-', 'MarkerSize', 4, 'MarkerFaceColor', [1, 1, 1]);
hold on;
for i = 1:length(samples)
    plot(fluence*100, 1000*sesdistance{i}, 'o-', 'Color', distinctColor(i-1), 'MarkerFaceColor', distinctColor(i-1), 'MarkerSize', 4);
end
hold off;
set(gca, 'XScale', 'Log');
set(gca, 'YLim', [749, 751]);
set(gca, 'XLim', [3e-2, 3e2]);
xlabel('Fluence [{\mu}J/cm^2]');
ylabel(sprintf('Arm length difference [mm], \\pm %.1f mm', 1000*ddistance));
grid on;
legend([{'HR'}, samples], 'Location', 'SE');
set(gcf, 'Name', folder);


% figure(25); %local fluence error bars
% s = 1;
% n = 1;
% plot(fluence(s:n:end)*100, 1:length(fluence(s:n:end)), 'bo', fluence_lower(s:n:end)*100, 1:length(fluence(s:n:end)), 'b+', fluence_upper(s:n:end)*100, 1:length(fluence(s:n:end)), 'b+');
% plot(fluence(s:n:end)*100, 1:length(fluence(s:n:end)), 'bo', [fluence_lower(s:n:end)*100, fluence_upper(s:n:end)*100]', [1;1]*(1:length(fluence(s:n:end))), 'b-+');
% set(gca, 'XScale', 'Log');
% grid on;


% figure(11); %phase investigation
% hold off;
% plot(fluence*100, (mod(atan2(shorty, shortx) + pi/2, 2*pi) - pi/2)./pi - 1, 'ro-');
% hold on;
% plot(fluence*100, (mod(atan2(shortfity, shortfitx) + pi/2, 2*pi) - pi/2)./pi - 1, 'r^:');
% plot(fluence*100, (mod(atan2(hrlongresy, hrlongresx) + pi/2, 2*pi) - pi/2)./pi, 'ko-');
% plot(fluence*100, (mod(atan2(hrlongresfity, hrlongresfitx) + pi/2, 2*pi) - pi/2)./pi, 'k^:');
% plot(fluence*100, (mod(atan2(seslongresy, seslongresx) + pi/2, 2*pi) - pi/2)./pi, 'bo-');
% plot(fluence*100, (mod(atan2(seslongresfity, seslongresfitx) + pi/2, 2*pi) - pi/2)./pi, 'b^:');
% hold off;
% set(gca, 'XScale', 'Log');
% grid on;
