#pragma rtGlobals=1		// Use modern global access method.

// Christian Walther, diploma thesis 2005

Menu "Macros"
	"Analyze Calibration Data File...", FitCalibration()
	"Apply Calibration to Measurement...", EvalCalibration()
End

Function Cos12(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ variable min = alog(logmin)
	//CurveFitDialog/ f(x) = log(1/2*( (1-n)*max+(1+n)*min + (max-min)*cos(2*pi*(x-x0)/p) + n*(max-min)*cos(4*pi*(x-x0)/p) ))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = logmin
	//CurveFitDialog/ w[1] = max
	//CurveFitDialog/ w[2] = n
	//CurveFitDialog/ w[3] = p
	//CurveFitDialog/ w[4] = x0

	variable min = alog(w[0])
	return 1/2*( (1-w[2])*w[1]+(1+w[2])*min + (w[1]-min)*cos(2*pi*(x-w[4])/w[3]) + w[2]*(w[1]-min)*cos(4*pi*(x-w[4])/w[3]) )
End

Function CosMM(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = 1/2*(max + min + (max-min)*cos(2*pi*(x-x0)/p))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 4
	//CurveFitDialog/ w[0] = min
	//CurveFitDialog/ w[1] = max
	//CurveFitDialog/ w[2] = p
	//CurveFitDialog/ w[3] = x0

	return 1/2*(w[1] + w[0] + (w[1]-w[0])*cos(2*pi*(x-w[3])/w[2]))
End

Function SatCosMM(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = min + (max - min)*(1 - exp(-sat*1/2*(1 + cos(2*pi*(x-x0)/p))))/(1 - exp(-sat))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = min
	//CurveFitDialog/ w[1] = max
	//CurveFitDialog/ w[2] = p
	//CurveFitDialog/ w[3] = x0
	//CurveFitDialog/ w[4] = sat

	return w[0] + (w[1] - w[0])*(1 - exp(-w[4]*1/2*(1 + cos(2*pi*(x-w[3])/w[2]))))/(1 - exp(-w[4]))
End

Function FitCalibration()
	Variable dummy
	String filepath
	
	Open /D /R /M="Choose a file saved by 'CW Calibrate Waveplate Adapt.vi'" dummy
	filepath = S_fileName
	if (strlen(filepath) == 0)
		return 0
	endif
	
	LoadWave /A /O /J /L={6, 7, 0, 0, 0} /B="N=angle;N=times;N=signal;" filepath
	Wave angle, times, signal
	
	LoadWave /A /O /J /L={0, 5, 1, 0, 1} /B="N=maxpower;" filepath
	Wave maxpower
	
	duplicate /O signal log_signal
	log_signal = log(signal)
	
	Make /O /D /N=4 cos_coef
	WaveStats /Q signal
	cos_coef[0] = 0 //min
	cos_coef[1] = V_max //max
	cos_coef[2] = 90 //p
	cos_coef[3] = angle(V_maxloc) //x0
	FuncFit /H="1000" CosMM cos_coef signal /X=angle // /D
	//Wave fit_signal
	
	duplicate /O signal var
	var = (signal - CosMM(cos_coef, angle))^2
	duplicate /O cos_coef var_coef
	redimension /N=5 var_coef
	var_coef[4] = 1
	FuncFit /H="00110" SatCosMM var_coef var /X=angle
	duplicate /O signal sig_stddev
	sig_stddev = sqrt(SatCosMM(var_coef, angle))
	
	Make /O /D /N=5 cos12_coef
	WaveStats /Q signal
	cos12_coef[0] = log(cos_coef[1]) - 6 //logmin
	cos12_coef[1] = cos_coef[1] //max
	cos12_coef[2] = 0 //n
	cos12_coef[3] = cos_coef[2] //p
	cos12_coef[4] = cos_coef[3] //x0
	Make /O /T cos12_constraints = {"K0 > -10", "K0 < log(cos_coef[1])", "K2 > -0.2", "K2 < 0.2"}
	FuncFit /H="00011" /L=500 Cos12 cos12_coef signal /X=angle /D /C=cos12_constraints /I=1 /W=sig_stddev
	Wave fit_signal
	
	duplicate /O sig_stddev errbandtop errbandbot logerrbandtop logerrbandbot bandtop bandbot logbandbot logbandtop
	errbandtop = sig_stddev
	errbandbot = -sig_stddev
	bandtop = Cos12(cos12_coef, angle) + sig_stddev
	bandbot = Cos12(cos12_coef, angle) - sig_stddev
	logerrbandtop = log(bandtop) - log(Cos12(cos12_coef, angle))
	logerrbandbot = log(bandbot) - log(Cos12(cos12_coef, angle))
	logbandtop = log(bandtop)
	logbandbot = log(bandbot)
	
	duplicate /O fit_signal log_fit_signal
	SetFormula log_fit_signal, "log(fit_signal)"
	
	Display /N=graph_lin signal vs angle as "Linear"; AppendToGraph fit_signal
	AppendToGraph bandtop vs angle; AppendToGraph bandbot vs angle
	ModifyGraph grid=2
	ModifyGraph mode(signal)=3,marker(signal)=19,msize(signal)=2,mrkThick(signal)=0,rgb(signal)=(65535,0,0)
	ModifyGraph mode(fit_signal)=0,rgb(fit_signal)=(0,0,65535)
	ModifyGraph mode(bandtop)=0,lstyle(bandtop)=1,rgb(bandtop)=(65535,43690,0),mode(bandbot)=0,lstyle(bandbot)=1,rgb(bandbot)=(65535,43690,0)
	Label bottom "Waveplate Angle []"
	Label left "Signal [V]"
	
	Display /N=graph_log log_signal vs angle as "Logarithmic"; AppendToGraph log_fit_signal
	AppendToGraph logbandtop vs angle; AppendToGraph logbandbot vs angle
	ModifyGraph grid=2
	ModifyGraph mode(log_signal)=3,marker(log_signal)=19,msize(log_signal)=2,mrkThick(log_signal)=0,rgb(log_signal)=(65535,0,0)
	ModifyGraph mode(log_fit_signal)=0,rgb(log_fit_signal)=(0,0,65535)
	ModifyGraph mode(logbandtop)=0,lstyle(logbandtop)=1,rgb(logbandtop)=(65535,43690,0),mode(logbandbot)=0,lstyle(logbandbot)=1,rgb(logbandbot)=(65535,43690,0)
	Label bottom "Waveplate Angle []"
	Label left "log\\B10\\M(Signal [V])"
	
	duplicate /O signal sigminusfit log_sigminusfit
	SetFormula sigminusfit, "signal - Cos12(cos12_coef, angle))"
	SetFormula log_sigminusfit, "log_signal - log(Cos12(cos12_coef, angle))"
	
	Display /N=graph_lin_error sigminusfit vs angle as "Linear Fit Error"
	AppendToGraph errbandtop vs angle
	AppendToGraph errbandbot vs angle
	ModifyGraph grid=2
	ModifyGraph mode=3,marker=19,msize=2,mrkThick=0,lstyle=1
	ModifyGraph mode(errbandtop)=0,rgb(errbandtop)=(65535,43690,0),mode(errbandbot)=0,rgb(errbandbot)=(65535,43690,0)
	Label bottom "Waveplate Angle []"
	Label left "Signal - Fit [V]"
	
	Display /N=graph_log_error log_sigminusfit vs angle as "Logarithmic Fit Error"
	AppendToGraph logerrbandtop vs angle
	AppendToGraph logerrbandbot vs angle
	ModifyGraph grid=2
	ModifyGraph mode=3,marker=19,msize=2,mrkThick=0,lstyle=1
	ModifyGraph mode(logerrbandtop)=0,rgb(logerrbandtop)=(65535,43690,0),mode(logerrbandbot)=0,rgb(logerrbandbot)=(65535,43690,0)
	Label bottom "Waveplate Angle []"
	Label left "log\\B10\\M(Signal) - log\\B10\\M(Fit)"
	
	Execute /Q "TileWindows /A=(2, 2) graph_lin, graph_lin_error, graph_log, graph_log_error"
End

Function EvalCalibration()
	Wave cos12_coef, maxpower
	Variable dummy
	Open /D /R /M="Choose measurement data file" dummy
	if (strlen(S_fileName) == 0)
		return 0
	endif
	LoadWave /A /O /J /L={2, 3, 0, 0, 1} /B="N=meas_angle;" S_fileName
	Wave meas_angle
	Redimension/N=(numpnts(meas_angle)-1) meas_angle
	duplicate /O meas_angle meas_power meas_power_stddev
	meas_power = maxpower[0]/cos12_coef[1]*Cos12(cos12_coef, meas_angle)
	meas_power_stddev = maxpower[0]/cos12_coef[1]*sqrt(SatCosMM(var_coef, meas_angle))
	Save /J /M="\n" meas_power, meas_power_stddev as "meas_power.txt"
End