% Christian Walther, diploma thesis 2005

function ls = loss(Ws, Ds)

    ts = zeros(length(Ws), length(Ds));
    tic;
    t = 0;
    
    for i = 1:length(Ws)
        for j = 1:length(Ds)
            fprintf('W = %4.2f, D = %4.2f...', Ws(i), Ds(j));
            ls(i, j) = singleloss(Ws(i), Ds(j));
            nt = toc;
            ts(i, j) = nt - t;
            fprintf(' %6.2f secs (%6.1f total)\n', nt - t, nt);
            t = nt;
        end
    end
    
    % graph calculation time
    figure(15);
    surf(Ds, Ws, ts);

    function l = singleloss(W, D)

        if (W == 0)
            l = (D > 1);
        elseif (D < 0.015*W^3) %for those parameters, the "small D" approximation (see below) is better than 1e-4
            l = 1 - exp(-4*D^2/(W^4*(exp(2/W^2) - 1)));
        else
            
            if (1.1*W^2 < D) %heuristic choice between the two approximations
                %for small w: integrate over half plane instead of circle
                approx = pi/4*W^2*(erf(sqrt(2)*(1-D)/W) + 1);
            else
                %for small d: quadratic approximation by a gaussian instead of a parabola (empirically, this improves the approximation)
                approx = pi/2*W^2*exp(-4*D^2/(W^4*(exp(2/W^2) - 1)))*(1 - exp(-2/W^2));
            end
            
            lapprox = 1 - approx/(pi/2*(1 - exp(-2/W^2))*W^2);
            if (lapprox < 1e-9)
                %quadrature takes very long for these points, and we're not really interested in them anyway
                l = lapprox;
            else
                l = 1 - dblquad(@polintegrand, 0, 1, 0, 2*pi, 1e-3*(pi/2*(1 - exp(-2/W^2))*W^2 - approx), @quadl) / (pi/2*(1 - exp(-2/W^2))*W^2);
            end
        end
        
        function g = polintegrand(r, phi)
            g = r.*exp(-2*(r.^2 + D^2 - 2*D*r.*cos(phi))/W^2);
        end

    end

end